import FluentAdFlowAdsWidget

@objc(FluentAdFlowWidgetViewManager)
class FluentAdFlowWidgetViewManager: RCTViewManager {

  override func view() -> (FluentAdFlowWidget) {
    return FluentAdFlowWidget()
  }

  @objc override static func requiresMainQueueSetup() -> Bool {
    return false
  }
}

class FluentAdFlowWidget : FluentAdFlowWidgetView {
  required init?(apiKey: String, referer: String) {
    super.init(apiKey: apiKey, referer: referer)
  }

  required init(frame: CGRect){
    super.init(frame: frame)
    super.events.listenTo(eventName: "onAdFlowContainerClosed", action: onAdFlowContainerClosed)
    super.events.listenTo(eventName: "onAdFlowContainerShow", action: onAdFlowContainerShow)
    super.events.listenTo(eventName: "onValidationFailed", action: onValidationFailed)
    super.events.listenTo(eventName: "onError", action: onError)
  }

  required init?(coder aDecoder: NSCoder) {
    fatalError("init(coder:) has not been implemented")
  }

  override func didSetProps(_ changedProps: [String]!) {
    print("changeProps: \(changedProps)")
    super.updateView(changedProps)
  }

  func onAdFlowContainerClosed(data: Any?) {
    print("-------- onAdFlowContainerClosed -------------")
    print(data)
    AdsViewEventEmitter.emitter.sendEvent(withName: "onAdFlowContainerClosed", body: ["onAdFlowContainerClosed"])
  }

  func onValidationFailed(data: Any?) {
    print("-------- on onValidationFailed -------------")
    print(data)
    AdsViewEventEmitter.emitter.sendEvent(withName: "onValidationFailed", body: ["onValidationFailed"])
  }

  func onError(data: Any?) {
    print("-------- on onError -------------")
    print(data)
    AdsViewEventEmitter.emitter.sendEvent(withName: "onError", body: ["onError"])
  }

  func onAdFlowContainerShow(data: Any?) {
    print("-------- on Show -------------")
    print(data)
    AdsViewEventEmitter.emitter.sendEvent(withName: "onAdFlowContainerShow", body: ["onAdFlowContainerShow"])
  }
}

@objc(AdsViewEventEmitter)
open class AdsViewEventEmitter: RCTEventEmitter {

  public static var emitter: AdsViewEventEmitter!

  override init() {
    super.init()
    AdsViewEventEmitter.emitter = self
  }

  @objc
  override public func supportedEvents() -> [String] {
    ["onReady", "onPending", "onError", "onValidationFailed", "onAdFlowContainerClosed", "onAdFlowContainerShow"] 
  }

  @objc override public static func requiresMainQueueSetup() -> Bool {
        return false
    }
}
